@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2010 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2010, 2011 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -152,7 +152,7 @@ class Engine(common.Engine):
         return iter(self._languages)
 
     @contextlib.contextmanager
-    def recognize(self, image, language, details=None):
+    def recognize(self, image, language, details=None, uax29=None):
         charset = 'iso-8859-15'
         if language == 'tur':
             charset = 'iso-8859-9'
