@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2010 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2010, 2011 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -64,7 +64,7 @@ def _test_from_file(base_filename, index):
                 rc = try_run(hocr2djvused.main, args)
         assert_equal(rc, 0)
         output = output_file.getvalue()
-    assert_ml_equal(output, expected_output)
+    assert_ml_equal(expected_output, output)
 
 def _rough_test_from_file(base_filename, args):
     args = ['#'] + shlex.split(args)
